/**
 * @typedef {import("../types/configuration/flattened.d.ts").FlattenedConfig} FlattenedConfig
 * @typedef {import("../types/configuration/flattened.d.ts").FlattenedConfigChecker} FlattenedConfigChecker
 * @typedef {import("../types/level.d.ts").default} Level
 * @typedef {import("../types/notice.d.ts").default} Notice
 * @typedef {import("../types/severity.d.ts").default} Severity
 * @typedef {import("../types/typeofformatter.d.ts").default} TypeofFormatter
 * @typedef {import("./formatter/formatter.js").default} Formatter
 * @typedef {import("./wrapper/wrapper.js").default} Wrapper
 */
export default class Metalint {
    /**
     * Crée une instance de Metalint à partir d'une configuration sur le système
     * de fichiers.
     *
     * @param {Object}          [options]           Les éventuelles options pour
     *                                              créer une instance de
     *                                              Metalint.
     * @param {string}          [options.config]    L'éventuel chemin vers le
     *                                              fichier de configuration de
     *                                              Metalint.
     * @param {boolean}         [options.fix]       L'éventuelle marque
     *                                              indiquant si les linters
     *                                              doivent corriger les
     *                                              fichiers.
     * @param {TypeofFormatter} [options.formatter] L'éventuel formatteur à
     *                                              utiliser.
     * @param {Level}           [options.level]     L'éventuel niveau minimum
     *                                              des notifications à
     *                                              rapporter.
     * @returns {Promise<Metalint>} Une promesse contenant l'instance de
     *                              Metalint.
     */
    static fromFileSystem(options?: {
        config?: string;
        fix?: boolean;
        formatter?: TypeofFormatter;
        level?: Level;
    }): Promise<Metalint>;
    /**
     * Crée une instance de Metalint.
     *
     * @param {FlattenedConfig} config       La configuration de Metalint.
     * @param {Object}          context      Le contexte.
     * @param {string}          context.root Le répertoire racine.
     */
    constructor(config: FlattenedConfig, { root }: {
        root: string;
    });
    /**
     * Vérifie (en appelant des linters) des répertoires et des fichiers.
     *
     * @param {string[]} bases Les répertoires et les fichiers.
     * @returns {Promise<Record<string, Notice[]|undefined>>} Une promesse
     *                                                        retournant la
     *                                                        liste des
     *                                                        notifications
     *                                                        regroupées par
     *                                                        fichier.
     */
    lintFiles(bases: string[]): Promise<Record<string, Notice[] | undefined>>;
    /**
     * Rapporte des résultats dans les formats choisis.
     *
     * @param {Record<string, Notice[]|undefined>} results La liste des
     *                                                     notifications
     *                                                     regroupées par
     *                                                     fichier.
     * @returns {Promise<Severity|undefined>} La sévérité la plus élevée des
     *                                        notifications ; ou
     *                                        <code>undefined</code> si les
     *                                        résultats ont aucune notification.
     */
    report(results: Record<string, Notice[] | undefined>): Promise<Severity | undefined>;
    #private;
}
export type FlattenedConfig = import("../types/configuration/flattened.d.ts").FlattenedConfig;
export type FlattenedConfigChecker = import("../types/configuration/flattened.d.ts").FlattenedConfigChecker;
export type Level = import("../types/level.d.ts").default;
export type Notice = import("../types/notice.d.ts").default;
export type Severity = import("../types/severity.d.ts").default;
export type TypeofFormatter = import("../types/typeofformatter.d.ts").default;
export type Formatter = import("./formatter/formatter.js").default;
export type Wrapper = import("./wrapper/wrapper.js").default;
